from django.contrib import admin, messages

from duck_django_soft_delete.admin.soft_deleted_at_filter import (
    SoftDeletedAtFilter,
)


class SoftDeleteAdmin(admin.ModelAdmin):
    list_filter = [SoftDeletedAtFilter, "created_at", "updated_at"]
    readonly_fields = [
        "deleted_at",
    ]
    ordering = [
        "-created_at",
    ]

    @admin.action()
    def soft_delete_selected(self, request, queryset):
        updated = 0
        for obj in queryset:
            if obj.deleted_at is None:
                obj.soft_delete()
                updated += 1
        self.message_user(
            request, f"{updated} soft-deleted items.", messages.SUCCESS
        )

    @admin.action()
    def restore_selected(self, request, queryset):
        updated = 0
        for obj in queryset:
            if obj.deleted_at is not None:
                obj.restore()
                updated += 1
        self.message_user(
            request, f"{updated} restaured items.", messages.SUCCESS
        )

    actions = [
        "soft_delete_selected",
        "restore_selected",
    ]

    def get_queryset(self, request):
        return self.model.everything.all()



from django.contrib.admin import SimpleListFilter


class SoftDeletedAtFilter(SimpleListFilter):
    title = "Soft Delete"
    parameter_name = "deleted"

    def lookups(self, request, model_admin):
        return [
            ("no", "Not Deleted"),
            ("yes", "Deleted"),
        ]

    def queryset(self, request, queryset):
        if self.value() == "no":
            return queryset.filter(deleted_at__isnull=True)

        elif self.value() == "yes":
            return queryset.filter(deleted_at__isnull=False)

        return queryset



from django.db import models


class NonSoftDeletedQuerySet(models.Manager):
    def get_queryset(self):
        queryset = super().get_queryset()
        return queryset.filter(deleted_at__isnull=True)



from duck_django_soft_delete.admin.soft_delete_admin import SoftDeleteAdmin
from duck_django_soft_delete.admin.soft_deleted_at_filter import (
    SoftDeletedAtFilter,
)

__all__ = [
    "SoftDeletedAtFilter",
    "SoftDeleteAdmin",
]



from typing import Type

from django.db import models
from django.db.models.query import Prefetch
from django.utils import timezone

from duck_django_soft_delete.queryset.non_soft_deleted_query_set import (
    NonSoftDeletedQuerySet,
)

class SoftDeleteTable(models.Model):
    """
    Abstract base model that provides soft delete functionality via a 'deleted_at' field.

    Models inheriting from this class will gain:
    - A soft delete method that timestamps 'deleted_at'.
    - A restore method to nullify 'deleted_at'.
    - Two managers:
        - `objects`: filters out soft-deleted records by default.
        - `everything`: returns all records, including soft-deleted ones.
    """

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    deleted_at = models.DateTimeField(null=True)

    everything = models.Manager()
    objects = NonSoftDeletedQuerySet()

    class Meta:
        abstract = True
        app_label = "duck_django_soft_delete"

    def soft_delete(self):
        """Marks the record as soft-deleted by setting the 'deleted_at' timestamp to now."""

        self.deleted_at = timezone.now()
        self.save(update_fields=["deleted_at"])

    def restore(self):
        """Restores a soft-deleted record by clearing the 'deleted_at' field."""

        self.deleted_at = None
        self.save(update_fields=["deleted_at"])

    @classmethod
    def build_prefetches(cls: Type["SoftDeleteTable"], relations: list[str]):
        prefetches = []

        for rel_name in relations:
            field = cls._meta.get_field(rel_name)
            related_model = getattr(field, "related_model", None)

            if (
                related_model
                and isinstance(related_model, type)
                and hasattr(related_model, "deleted_at")
                and issubclass(related_model, SoftDeleteTable)
            ):
                qs = related_model.objects.filter(deleted_at__isnull=True)
                prefetches.append(Prefetch(rel_name, queryset=qs))
            else:
                prefetches.append(rel_name)

        return prefetches
